/* filename: loaddata.g
** description: reads time series data from ASCII data set into
** Gauss variable SERIES
** created:  12/10/01
**
*/

if datasel > 7;

   /* USE SIMULATED DATA
   */

   lsimdata  = datapath $+ "\\" $+ lmodel $+ "sim" $+ dataselstr;
   open fhsimdata = ^lsimdata for read;
   series_sim = readr(fhsimdata,Tsim[datasel]);
   closeall fhsimdata;   
   
   series = series_sim;
   ti = seqa(1,1,Tsim[datasel]);

elseif datasel > 3; 

   /* USE ACTUAL DATA, OUTPUT LINEAR TREND EXTRACTED
   **
   ** Time series, extracted 10/01
   ** Filename: data_tr1.prn (linear trend extracted)
   ** 1. Log real GDP, per capita, GDPQ_CLDTR
   ** 2. Inflation, CPI-U: INFL, quarter-to-quarter, annualized
   ** 3. Federal Funds Rate: FYFF, annualized 
   **
   ** Quarterly data from 1960:I to 1997:4
   ** 152 Observations
   */

   load path   = ^datapath series[152,3]= "data_tr1.prn";
   ti          = seqa(1960,0.25,152);
   
   /* Subperiods in CGG
   ** Pre-Volcker:       1960:1 to 1979:2, obs  1 -  78
   ** Volcker-Greenspan: 1979:3 to 1997:4, obs 79 - 152
   ** post-82:           1982:4 to 1997:4, obs 92 - 152
   */
   if datasel == 4;
      /* Full Sample */
   
   elseif datasel == 5;
      /* Pre-Volcker Period */
      series = series[1:78,.];
      ti     = ti[1:78,.];
      
   elseif datasel == 6;
      /* Volcker-Greenspan */
      series = series[79:152,.];
      ti     = ti[79:152,.];
      
   elseif datasel == 7;
      /* post-1982 */
      series = series[92:152,.];
      ti     = ti[92:152,.];

   endif;
   
else; 

   /* USE ACTUAL DATA, OUTPUT HP-filtered
   **
   ** Time series, extracted 10/01
   ** Filename: data_hp1.prn (hp-filtered)
   ** 1. Log real GDP, per capita, GDPQ_CLDHP
   ** 2. Inflation, CPI-U: INFL, quarter-to-quarter, annualized
   ** 3. Federal Funds Rate: FYFF, annualized 
   **
   ** Quarterly data from 1960:I to 1997:4
   ** 152 Observations
   */

   load path   = ^datapath series[152,3]= "data_hp1.prn";
   ti          = seqa(1960,0.25,152);
   
   /* Subperiods in CGG
   ** Pre-Volcker:       1960:1 to 1979:2, obs  1 -  78
   ** Volcker-Greenspan: 1979:3 to 1997:4, obs 79 - 152
   ** post-82:           1982:4 to 1997:4, obs 92 - 152
   */
   if datasel == 0;
      /* Full Sample */
   
   elseif datasel == 1;
      /* Pre-Volcker Period */
      series = series[1:78,.];
      ti     = ti[1:78,.];
      
   elseif datasel == 2;
      /* Volcker-Greenspan */
      series = series[79:152,.];
      ti     = ti[79:152,.];
      
   elseif datasel == 3;
      /* post-1982 */
      series = series[92:152,.];
      ti     = ti[92:152,.];

   endif;
   
endif;
