/* filename:    detsize.g
** description: Computes size of determinacy region based on
**              output of prior simulator
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

#include c:\projects\active\sunspots\gauss1\pathspec.g;
#include c:\projects\active\sunspots\gauss1\sunspec.g;

/******************************************************************
**         Load determinacy indicators 
*/

mhrun  = "0";
lpath  = resupath $+ "\\mhrun" $+ mhrun ;

if mhrun $== "0";
   dataselstr = "0";
endif;

ldraw  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "dt";
open fhdraw = ^ldraw for read;

nburn       = 1;        /* Number of initial draws to be discarded */

drawrow = readr( fhdraw, nburn); 
drawdim = cols(drawrow);
drawrow = seekr( fhdraw, nburn);

/* Part 1: Compute the mean of x(i) and x^2(i)
*/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);
drawcross  = zeros(drawdim,drawdim);

eofloop = 0;
nblock  = 5000;
ndraws  = 0;

do until eofloop; 

   drawblock = readr( fhdraw, nblock );

   drawmean  = drawmean    + sumc(drawblock)';
   drawsqmean = drawsqmean + sumc(drawblock^2)';
   drawcross  = drawcross  + drawblock'*drawblock;  

   ndraws = ndraws + rows(drawblock);

   eofloop = eof(fhdraw);

endo;

drawmean   = drawmean/ndraws;
drawsqmean = drawsqmean/ndraws;
drawstdd   = sqrt(drawsqmean - (drawmean)^2);
drawcov    = drawcross/ndraws - drawmean'*drawmean;

/* Report Prior size of Determinacy Regions and MC S.E.
**
*/
"MODEL   " lmodel;
"PRIOR   " lprior;
"MH-Run  " mhrun;
"Regime Shift" subT;
"=============================================";
" " ;

statlabel = { DET1 IND1 };
outmat = statlabel' ~ drawmean' ~ (drawstdd'/sqrt(ndraws)) ;
let mask[1,3] = 0 1 1;
let fmt[3,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,mask,fmt);

end;
