/* filename:    sundet.g
** description: The program checks wheter the DSGE parameters
**              are in the regions: non-existence, determinacy, indeterminacy
** created:     12/15/01
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\sunspots\gauss1\pathspec.g
#include c:\projects\active\sunspots\gauss1\sunspec.g

/* Open files with Parameter Draws
*/
mhrun  = "0";
lpath  = parapath $+ "\\mhrun" $+ mhrun;  

/* If mhrun is zero then set dataselstr and subTstr to zero
*/
if mhrun $== "0";
   dataselstr = "0";
endif;

lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pa";
open fhpara = ^lpara for read;

/* Define some Constants
*/
nsim   = 5000;   /* block size */

/* Initialize Output files
*/
opath = resupath $+ "\\mhrun" $+ mhrun;
odet  = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "dt";

create fhdet = ^odet with DET, 2, 8;

/* Initialization
*/ 
eofloop  = 0;
loopct   = 0;
ninvalid = 0;

do until eofloop; 

   /* Read blocks of size nsim
   */
   parasim = readr( fhpara, nsim );
   npara = cols(parasim);
   nread = rows(parasim);

   invaliddraw = zeros(nread,1);      
   detsim      = zeros(nread,2);

   j = 1;
   do until j > rows(parasim);

      /* For each element of the block, compute the desired statistic
      */

      {T1,TC,TEPS,TETA,RC} = dsgesolv(parasim[j,.]',mspec);
     
      if (RC[1] == 1) AND (RC[2]==0);
         /* indeterminacy */
         detsim[j,2] = 1;
         
      elseif (RC[1] == 1) AND (RC[2]==1); 
         /* determinacy */
         detsim[j,1] = 1; 
       
      else;
         invaliddraw[j,1] = 1;  

      endif; 

      j = j+1;
  endo;

  ninvalid = ninvalid + sumc(invaliddraw);

  /* Write Output
  */
  wr = writer(fhdet,detsim);
  
  loopct = loopct + 1;
  locate 1,1;
  "Loop            " loopct;
  "Size of Regions " meanc(detsim);
  eofloop = eof(fhpara);
  
endo;

"Number of Invalid Draws:" ninvalid;

closeall fhdet, fhpara;
#include c:\projects\active\sunspots\gauss1\sunmod.src;