/*******************************************************/
/*                                                     */
/* Computing Moments based on parameters of DSGE model */
/*                                                     */
/*******************************************************/

/* filename:    sunpac.g
** description: The program converts the DSGE parameters
**              into unconditional moments
** created:     21/10/99
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\sunspots\gauss1\pathspec.g;
#include c:\projects\active\sunspots\gauss1\sunspec.g;

nac = 15;
nvd = 3*4;

/* Open files with Parameter Draws
*/
mhrun  = "2";

if mhrun $== "0";
   dataselstr = "0";
endif;

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;

lpara  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pa";
open fhpara = ^lpara for read;

/* Define some Constants
*/
nblock  = 1000;    /* block size */
nuse    = 5;      /* use every nuse observation */

/* Initialize Output files
*/
oac  = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "ac";
ovd  = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "vd";

create fhac = ^oac with AC, nac, 8;
create fhvd = ^ovd with VD, nvd, 8;
 
/* Initialization
*/ 
eofloop = 0;
loopct  = 1;

do until eofloop; 

   /* Read blocks of size nblock
   */
   parasim = readr( fhpara, nblock );
   nread = rows(parasim);

   indseq = seqa(1,1,nread);
   indseq = indseq % nuse;
   indseq = indseq .NE 0;
 
   parasim = delif(parasim,indseq);
   nread = rows(parasim);

   acsim = zeros(nread,nac);
   vdsim = zeros(nread,nvd);
   
   j = 1;
   do until j > nread;
   
      /* For each element of the block, compute the desired statistic
      */
      
      para = parasim[j,.]';

      {GAM0_r, GAM1_r, GAM0_g, GAM1_g,
       GAM0_a, GAM1_a, GAM0_e, GAM1_e }
        = sunmom(para);

      GAM0 = GAM0_r + GAM0_g + GAM0_a + GAM0_e;
      GAM1 = GAM1_r + GAM1_g + GAM1_a + GAM1_e;
      
      /* Variance Decompositions */
      vd   = (diag(GAM0_r)./diag(GAM0)) ~ (diag(GAM0_g)./diag(GAM0))
           ~ (diag(GAM0_a)./diag(GAM0)) ~ (diag(GAM0_e)./diag(GAM0));
      vdsim[j,.] = vec(vd')';
      
      /* Autocovariances */
      CORR0  = corrvc(GAM0);
      Stddev = sqrt(diag(GAM0));
      CORR1  = GAM1 ./ (Stddev*Stddev');
      
      acsim[j,.] = Stddev' ~ CORR0[1,2:3] ~ CORR0[2,3] ~ vec(CORR1')';

      j = j+1;

   endo;

   /* Write Output
   */
   wr = writer(fhac,acsim);
   wr = writer(fhvd,vdsim);

   locate 1,1;
   "Block" loopct;
   "Draws Processed:" rows(acsim);   

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;


closeall fhpara, fhac, fhvd;

/*******************************************************************
*/

proc (8) = sunmom(para);
local sig_cov, sig_chol, omega, rotmat, covpara,
      TT, TC, TEPS, TETA, RC,
      DD, RR, VV, HH, QQ, ZZ, nstate,
      GAM0s, GAM1s,
      GAM0_r, GAM1_r, GAM0_g, GAM1_g,
      GAM0_a, GAM1_a, GAM0_e, GAM1_e;
      
/* Construct Chol decomp of Covariance Matrix 
*/ 
covpara = para[10:17];

omega = zeros(4,4);
omega[1,1] = covpara[5];
omega[2,2] = covpara[6];
if covpara[6] > 0; 
   omega[3,2] = covpara[1]*covpara[7];
else;
   omega[3,2] = 0;
endif;
omega[3,3] = sqrt( covpara[7]^2-omega[3,2]^2);
omega[4,4] = covpara[8];

rotmat = ( 1 | 0 | 0 | covpara[2] ) ~
         ( 0 | 1 | 0 | covpara[3] ) ~
         ( 0 | 0 | 1 | covpara[4] ) ~
         ( 0 | 0 | 0 | 1 );

sig_chol = rotmat*omega;

/* solve the DSGE model
*/
{TT,TC,TEPS,TETA,RC} = dsgesolv(para,mspec);
nstate = cols(TT);

if (RC[1] == 1) AND (RC[2]==1);
   /* determinacy */
   RR = TEPS ~ zeros(nstate,1);
elseif (RC[1] == 1) AND (RC[2]==0) ;
   /* indeterminacy */
   RR = TEPS ~ TETA;
endif;
     
ZZ      = zeros(3,nstate);
ZZ[1,1] = 1; 
ZZ[2,2] = 4; /* measured inflation rates are annualized */
ZZ[3,3] = 4; /* measured interest rates are annualized */

HH = zeros(3,3);
VV = zeros(4,3);

QQ = sig_chol[.,1]*sig_chol[.,1]';  
GAM0s  = real(dlyap(TT,RR*QQ*RR'));
GAM0_r = ZZ*GAM0s*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH;
GAM1_r = ZZ*(TT*GAM0s)*ZZ' + ZZ*(TT*RR*VV);

QQ = sig_chol[.,2]*sig_chol[.,2]';  
GAM0s  = real(dlyap(TT,RR*QQ*RR'));
GAM0_g = ZZ*GAM0s*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH;
GAM1_g = ZZ*(TT*GAM0s)*ZZ' + ZZ*(TT*RR*VV);

QQ = sig_chol[.,3]*sig_chol[.,3]';  
GAM0s  = real(dlyap(TT,RR*QQ*RR'));
GAM0_a = ZZ*GAM0s*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH;
GAM1_a = ZZ*(TT*GAM0s)*ZZ' + ZZ*(TT*RR*VV);

QQ = sig_chol[.,4]*sig_chol[.,4]';  
GAM0s  = real(dlyap(TT,RR*QQ*RR'));
GAM0_e = ZZ*GAM0s*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH;
GAM1_e = ZZ*(TT*GAM0s)*ZZ' + ZZ*(TT*RR*VV);

retp( GAM0_r, GAM1_r, GAM0_g, GAM1_g,
      GAM0_a, GAM1_a, GAM0_e, GAM1_e );

endp;
      
#include c:\projects\active\sunspots\gauss1\sunmod.src;
#include c:\projects\active\sunspots\gauss1\sunlh.src;
