/*******************************************************/
/*                                                     */
/* Computing Impulse responses based on                */
/* parameters of DSGE models                           */
/*                                                     */
/*******************************************************/

/* filename:    sunpirf.g
** description: The program converts the DSGE parameters
**              into impulse response functions
** created:     21/10/99
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\sunspots\gauss1\pathspec.g;
#include c:\projects\active\sunspots\gauss1\sunspec.g;

nirf = 20;

/* Open files with Parameter Draws
*/
mhrun  = "2";

if mhrun $== "0";
   dataselstr = "0";
endif;

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;

lpara  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pa";
open fhpara = ^lpara for read;

/* Define some Constants
*/
nblock  = 1000;  /* block size */
nuse    = 5;     /* use every nuse observation */

/* Initialize Output files
*/
oirf = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "ir";
create fhirf = ^oirf with IRF, 12*nirf, 8;
       
/* Initialization
*/ 
eofloop = 0;
loopct  = 1;

do until eofloop; 

   /* Read blocks of size nblock
   */
   parasim = readr( fhpara, nblock );
   nread = rows(parasim);

   indseq = seqa(1,1,nread);
   indseq = indseq % nuse;
   indseq = indseq .NE 0;
 
   parasim = delif(parasim,indseq);
   nread = rows(parasim);

   irfsim = zeros(nread,12*nirf);
   
   j = 1;
   do until j > nread;
   
      /* For each element of the block, compute the desired statistic
      */
      para = parasim[j,.]';

      ssirf = sunirf(para,nirf);
      
      irfsim[j,.] = vec(ssirf)';
      
      j = j+1;
      
   endo;

   /* Write Output
   */
   wr = writer(fhirf,irfsim);

   locate 1,1;
   "Block" loopct;
   "Draws Processed:" nread;   

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara, fhirf;

/*******************************************************************
*/

proc (1) = sunirf(para,nirf);
/* INPUT:
**   para   : parameter vector 
**   nirf   : length of irf
** OUTPUT:
**   ssirf  : impulse response of [y,inf,r2] to 
**            the structural and sunspot shocks.
**            0 if retcode=0
**            (nirf by 3*4)
*/
local sig_cov, sig_chol, impact, covpara, omega, rotmat,
      T1,TC,T0,TETA,RC,RR,
      ssirfall, ssirf, sh_ind, t;

/* Construct Chol decomp of Covariance Matrix 
*/ 
covpara = para[10:17];
covpara[1] = 0;

omega = zeros(4,4);
omega[1,1] = covpara[5];
omega[2,2] = covpara[6];
if covpara[6] > 0; 
   omega[3,2] = covpara[1]*covpara[7];
else;
   omega[3,2] = 0;
endif;
omega[3,3] = sqrt( covpara[7]^2-omega[3,2]^2);
omega[4,4] = covpara[8];

rotmat = ( 1 | 0 | 0 | covpara[2] ) ~
         ( 0 | 1 | 0 | covpara[3] ) ~
         ( 0 | 0 | 1 | covpara[4] ) ~
         ( 0 | 0 | 0 | 1 );

sig_chol = rotmat*omega;

/* solve DSGE
*/
{T1,TC,T0,TETA,RC} = dsgesolv(para,mspec);

if (RC[1] == 1) AND (RC[2]==1);
   /* determinacy */
   RR = T0 ~ zeros(rows(T1),1);
elseif (RC[1] == 1) AND (RC[2]==0) ;
   /* indeterminacy */
   RR = T0 ~ TETA;
endif;

ssirfall = zeros(nirf,12); 

sh_ind = 1;
do until sh_ind > 4;
   
   impact = sig_chol[.,sh_ind];
   ssirf  = zeros( nirf, rows(T1) );
   
   /* compute IRF for each orthogonalized shock
   */
   ssirf[1,.] = (RR*impact)';
   t = 2;
   do until t > nirf;
      ssirf[t,.] = (T1*ssirf[t-1,.]')';
      t = t+1;
   endo;

   ssirfall[.,1+3*(sh_ind-1):3*sh_ind] = ssirf[.,1:3] .* (1 ~ 4 ~ 4);
   /* measured inflation and interest rates are annualized */

   sh_ind = sh_ind+1;
   
endo;

retp(ssirfall);
endp;
      
#include c:\projects\active\sunspots\gauss1\sunmod.src;
#include c:\projects\active\sunspots\gauss1\sunlh.src;
