/* filename:    sunprio2.g
** description: The program converts the draws from the prior distribution
**              and deletes the ones that lie outside of the parameter space
** created:     12/15/01
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\sunspots\gauss1\pathspec.g
#include c:\projects\active\sunspots\gauss1\sunspec.g

/* Open files with Parameter Draws
*/
mhrun  = "0";
lpath  = parapath $+ "\\mhrun" $+ mhrun;  

/* If mhrun is zero then set dataselstr to zero
*/
if mhrun $== "0";
   dataselstr = "0";
endif;

lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pa0";
open fhlpara = ^lpara for read;

/* Define some Constants
*/
nsim   = 5000;   /* block size */

/* Initialize Output files
*/
opath = lpath;
opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pa";
create fhopara = ^opara with PARAM, npara, 8;

opath = resupath $+ "\\mhrun" $+ mhrun;
odet  = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "dt";
create fhdet = ^odet with DET, 2, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 0;
nread   = 0;
nvalid  = 0;

do until eofloop; 

   /* Read blocks of size nsim
   */
   parasim = readr( fhlpara, nsim );
   nread   = nread + rows(parasim);

   /* Eliminate draws that are in the non-existence region
   */
   invaliddraw = zeros(rows(parasim),1);
   detsim      = zeros(rows(parasim),2);

   j = 1;
   do until j > rows(parasim);

      /* For each element of the block, compute the desired statistic
      */

      /* Regime 1 */
      {T1,TC,TEPS,TETA,RC} = dsgesolv(parasim[j,.]',mspec);
     
      if (RC[1] == 1) AND (RC[2]==0);
         /* indeterminacy */
         detsim[j,2] = 1;
         
      elseif (RC[1] == 1) AND (RC[2]==1); 
         /* determinacy */
         detsim[j,1] = 1; 
       
      else;
         invaliddraw[j,1] = 1;  

      endif; 

      j = j+1;
  endo;

  parasim = delif(parasim, invaliddraw);
  detsim  = delif(detsim, invaliddraw);
  nvalid  = nvalid + rows(parasim);

  /* Write Output
  */
  wr = writer(fhdet,detsim);
  wr = writer(fhopara,parasim);
  
  loopct = loopct + 1;
  locate 1,1;
  "Loop            " loopct;
  "Size of Regions [det R1, indet R1]"; meanc(detsim)';
  eofloop = eof(fhlpara);
  
endo;

"Fraction of valid draws" nvalid/nread;

closeall fhdet, fhlpara, fhopara;
#include c:\projects\active\sunspots\gauss1\sunmod.src;
#include c:\projects\active\sunspots\gauss1\sunlh.src;
