/*************************************************************************
**  Solve the DSGE model via GENSYS
**************************************************************************/


/* filename: suntest.g
** created:  10/18/01
** Modifications:
**
*/
new;
cls;
library pgraph, user;
_fcmptol = 1E-10;

dlibrary -a PszTgSen,PsdTgSen;

#include c:\projects\active\sunspots\gauss2\sunmod.src


/* Select Policy Rule
** msel = 1: curent inflation
** msel = 2: future inflation
*/
   msel = 1;
   
/* Define model parameters
** Note: ph1 is lower case phi, ph2 is upper case phi
*/
            
   psi1   = 0.9; /*0.65422;  */  /* inflation elasticity */
   psi2   = 0;    /* output elasticity */
   rhor   = 0;    /* interest rate smoothing */
   pista  = 4;
   rsta   = 1.66763;  /* annualized real interest rate [percent] */
   kap    = 0.3;      /* slope of Phillips Curve */
   tauinv = 0.5;      /* relative risk aversion */
   om     = 0.9; 
   rhog   = 0.8;    
   rhoa   = 0.7;
   rgz    = 0.39844;
   rer    = 0.23734;
   reg    = 0.06297;
   rez    = 0.11842;
   sigr   = 0.27660;
   sigg   = 0.22955;
   sigz   = 1.80696;
   sige   = 0.01629;

bet  = 1/(1+rsta/100)^(1/4);
tau = 1/tauinv;

psi1 = 1+(kap*tau+1-bet)^2/(4*kap*tau*bet)-1E-15;
psi1 = 0.9;

l1  = 0.5*(1 + (kap*tau+1)/bet);
l2  = 0.5*sqrt( ((kap*tau+1)/bet-1)^2 + 4*kap*tau*(1-psi1)/bet );


rhog = 0.9;

   para  = psi1 | psi2  | rhor | pista | 
           rsta | kap   | tauinv | om | rhog | rhoa |
           rgz  | rer   | reg   | rez  | 
           sigr | sigg  | sigz  | sige ;

"Parameters"; para';
"Specification"; msel;

/* Print System Matrices to Screen
*/
{T1,TC,T0,TETA,RC} = dsgesolv(para,msel);

format /mb1 /rds 10,5;

"IRF - 0"; T0;
"IRF - 1"; T1*T0;
"Sunspot Impact"; TETA;

/********************************************************************
** Plot Impulse Response Function
********************************************************************/

nirf = 20;
ti = seqa(1,1,nirf);

titlestr = "Monetary Shock"  $|
           "Supply Shock" $|
           "Demand Shock" $|
           "Sunspot Shock";

ystr     = "Output" $|
           "Inflation" $|
           "Exp. Output" $|
           "Exp. Inflation" $|
           "Nom Interest";

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
          
begwind;
window(4,3,0);

shockind = 1;

do until shockind > 4;

   impulse = zeros(4,1);
   impulse[shockind] = 1;

   /* simulate IRFs
   */
   {ssirf,retcode} = dsgeirf(para,msel,nirf,impulse);

   "Shock" Shockind;

   if retcode == 0;
      "No equilibrium exists";
      "";
   elseif retcode == 1;
      "Unique equilibrium";
      "";
   elseif retcode == 2;
      "Sunspots";
      "";
   endif;


   if shockind == 1;
      title("Output");
   else;
      title("");
   endif;

   ylabel(titlestr[shockind]);
   xy(ti,ssirf[.,1 4]);
   ylabel("");

   nextwind;

   if shockind == 1;
      title("Inflation");
   else;
      title("");
   endif;

   xy(ti,ssirf[.,2 5]);
   nextwind;

   if shockind == 1;
      title("Interest");
   else;
      title("");
   endif;

   xy(ti,ssirf[.,3]);
   nextwind;
    
   noplot:

   shockind = shockind+1;

endo;
endwind;
end;
